SystemOrganization addCategory: #LostInMorvanChat!
SystemOrganization addCategory: 'LostInMorvanChat-Client'!
SystemOrganization addCategory: 'LostInMorvanChat-Message'!
SystemOrganization addCategory: 'LostInMorvanChat-Server'!

ComposableModel subclass: #ChatConsole
	instanceVariableNames: 'chat list input'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatConsole commentStamp: 'FrancoisJarriges 6/2/2016 15:00' prior: 0!
I represent the console window to be used for the chat application to write and read messages.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	chat:		<Object>
	input:		<Object>
	list:		<Object>


    Implementation Points!

!ChatConsole class methodsFor: 'api' stamp: 'FrancoisJarriges 6/1/2016 11:40'!
attach: aChatClient

	| aChatConsole |
	
	aChatConsole := self new chat: aChatClient.
	aChatConsole openWithSpec whenClosedDo: [ aChatClient disconnect ].
	^ aChatConsole! !

!ChatConsole class methodsFor: 'api' stamp: 'FrancoisJarriges 6/1/2016 11:53'!
defaultSpec
	<spec: #default>
	
	^ SpecLayout composed
			newColumn: [ :c |
				c add: #list; add: #input height: 30 ]; yourself
! !

!ChatConsole methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 11:32'!
chat: aChatClient
	chat := aChatClient! !

!ChatConsole methodsFor: 'api' stamp: 'FrancoisJarriges 6/2/2016 10:56'!
initializeWidgets

	list := ChatConsoleListModel new.
	input := TextInputFieldModel new
		ghostText: 'Type your message here...';
		enabled: true;
		acceptBlock: [ :inputString |
			chat send: inputString.
			input text: '' ].
	self focusOrder add: input
! !

!ChatConsole methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 11:31'!
input
	^ input! !

!ChatConsole methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 11:31'!
list
	^ list! !

!ChatConsole methodsFor: 'api' stamp: 'FrancoisJarriges 6/2/2016 12:57'!
print: aCollectionOfChatMessages

	list addItems: (aCollectionOfChatMessages collect: [ :eachChatMessage | 
		eachChatMessage ])
! !

!ChatConsole methodsFor: 'api' stamp: 'FrancoisJarriges 6/2/2016 17:08'!
title
	^ chat login, ' on ', chat url! !

ListModel subclass: #ChatConsoleListModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatConsoleListModel commentStamp: 'FrancoisJarriges 6/2/2016 15:06' prior: 0!
I specialize the standard UI ListModel class for a chat console window.


    Implementation Points!

!ChatConsoleListModel methodsFor: 'api' stamp: 'FrancoisJarriges 6/2/2016 12:55'!
addItems: aList
	"Add a new list of items to the existing list."

	| oldContents |
	oldContents := listHolder value.
	listHolder value: {oldContents . (aList sorted: sortingBlockHolder value)} concatenation.
	listAnnouncer announce: (ValueChanged oldValue: oldContents newValue: listHolder value).
	! !

Object subclass: #ChatClient
	instanceVariableNames: 'url login exit messages console lastMessageIndex networkError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Client'!

!ChatClient commentStamp: 'FrancoisJarriges 6/2/2016 15:05' prior: 0!
I am a chat client. I'm sending my own messages  to an HTTP server and getting all the messages sent by all the chat clients including me to display on a console. 

ChatClient connect: 'localhost' port: 8080 login: 'toto'.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	console:		<Object>
	exit:		<Object>
	lastMessageIndex:		<Object>
	login:		<Object>
	messages:		<Object>
	networkError:		<Object>
	url:		<Object>


    Implementation Points!

!ChatClient class methodsFor: 'public' stamp: 'FrancoisJarriges 6/1/2016 11:19'!
connect: aHost port: aPort login: aLogin

	^ self new
		host: aHost port: aPort login: aLogin;
		start
! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/3/2016 15:40'!
basicSecuredGet: commandString
	
	| response |
	
	[ response := ZnClient new 
		url: commandString; 
		get ] 
		on: NetworkError 
		do: [ :thisNetworkError | 
			self printConnectionErrorMessage: thisNetworkError asString.
			response := nil ].
	response ifNotNil: [ networkError := nil ].
	^ response 
! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/3/2016 15:40'!
basicSecuredPost: commandString message: newChatMessage
	| response |
	
	[ response := ZnClient new
		url: commandString;
		entity: (ZnEntity 
              with: (STON toString: newChatMessage)
              type: ZnMimeType applicationSton);
		post ]
		on: NetworkError 
		do: [ :thisNetworkError | 
			self printConnectionErrorMessage: thisNetworkError asString.
			response := nil ].
	response ifNotNil: [ networkError := nil ].
	^ response! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 10:30'!
command: aPath

	^ '{1}{2}' format: { url . aPath }! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 10:30'!
command: aPath argument: anArgument

	^ '{1}{2}{3}' format: { url . aPath . anArgument asString }! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/3/2016 13:16'!
commandAddMessage

	^ self command: '/messages/add'! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 10:34'!
commandLastMessageIndex

	^ self command: '/messages/count'! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/1/2016 14:31'!
commandMessagesFromLastIndex

	^ self command: '/messages/' argument: lastMessageIndex ! !

!ChatClient methodsFor: 'public' stamp: 'FrancoisJarriges 6/2/2016 16:32'!
disconnect

	self postNewMessage: (ChatMessage from: '- chat server -' text: login, ' exited from the chat room').
	exit := true

! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/3/2016 12:32'!
getLastMessageIndexAtStart

	| response |	
			
	[ response := self basicSecuredGet: self commandLastMessageIndex ] value
		ifNil: [ ^ lastMessageIndex ].
	^ self objectFromSTON: response
! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/3/2016 15:39'!
getNewMessages
	"Gets the new messages that have been posted since the last request."
	"Prints the new messages on the console."

	| response receivedChatMessages |
	
	^ [ response := self basicSecuredGet: self commandMessagesFromLastIndex ] value
		ifNil: [ 0 ]
		ifNotNil: [
			receivedChatMessages := self objectFromSTON: response.
			console print: receivedChatMessages.
			messages addAll: receivedChatMessages.
			receivedChatMessages size ]
	
! !

!ChatClient methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/1/2016 11:22'!
host: aHost port: aPort login: aLogin

	url := 'http://' , aHost , ':' , aPort asString.
	login := aLogin

! !

!ChatClient methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/2/2016 15:32'!
initialize

	super initialize.
	exit := false.
	networkError := nil.
	lastMessageIndex := 0.
	messages := OrderedCollection new! !

!ChatClient methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/2/2016 17:04'!
login
	^ login! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/3/2016 12:41'!
objectFromSTON: aByteString
	^ (STONReader on: aByteString readStream) next
! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/3/2016 13:05'!
postNewMessage: aChatMessage

	^ self basicSecuredPost: self commandAddMessage
		message: aChatMessage! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/2/2016 16:32'!
printConnectionErrorMessage: aNetworkErrorString

	| errorMessage |
	
	aNetworkErrorString = networkError ifFalse: [ 
		networkError := aNetworkErrorString.
		errorMessage := ChatMessage 
			from: '- chat client -' 
			text: 'Network Error :', networkError.
		console print: errorMessage asOrderedCollection ]


	! !

!ChatClient methodsFor: 'private' stamp: 'FrancoisJarriges 6/2/2016 11:39'!
refreshMessages
	"Create and schedule a light process to get new posted messages every 2 seconds."
	"At client disconnection the process ends."
	
	[
		[ exit ] whileFalse: [
			(Delay forSeconds: 2) wait.
			lastMessageIndex := lastMessageIndex + self getNewMessages ] 
	] fork! !

!ChatClient methodsFor: 'public' stamp: 'FrancoisJarriges 6/2/2016 11:37'!
send: aString
	"Post a message to the server and get back from it the list of all new posted messages."
	
	| newChatMessage |
	
	newChatMessage := ChatMessage from: login text: aString.
	self postNewMessage: newChatMessage.
	lastMessageIndex := lastMessageIndex + self getNewMessages
! !

!ChatClient methodsFor: 'public' stamp: 'FrancoisJarriges 6/2/2016 16:32'!
start

	console := ChatConsole attach: self.
	lastMessageIndex := self getLastMessageIndexAtStart.
	self postNewMessage: (ChatMessage from: '- chat server -' text: login, ' joined the chat room').
	self refreshMessages! !

!ChatClient methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/2/2016 17:04'!
url
	^ url! !

Object subclass: #ChatMessage
	instanceVariableNames: 'sender text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Message'!

!ChatMessage commentStamp: 'FrancoisJarriges 6/2/2016 14:57' prior: 0!
I'm a simple chat message. I identify my sender and my text.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	sender:		<Object>
	text:		<Object>


    Implementation Points!

!ChatMessage class methodsFor: 'instance-creation' stamp: 'FrancoisJarriges 6/1/2016 08:59'!
from: aSender text: aText

	^ self new sender: aSender; text: aText; yourself
! !

!ChatMessage class methodsFor: 'api' stamp: 'FrancoisJarriges 6/1/2016 12:54'!
separator
	^ '>'! !

!ChatMessage methodsFor: 'printing' stamp: 'FrancoisJarriges 6/1/2016 15:01'!
printOn: aStream

	aStream 
		<< self sender; 
		<< self class separator;
		<< String space;
		<< self text; 
		<< String crlf! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 08:57'!
sender
	^ sender! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 08:57'!
sender: anObject
	sender := anObject! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 08:57'!
text
	^ text! !

!ChatMessage methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 08:57'!
text: anObject
	text := anObject! !

Object subclass: #ChatMessagesQueue
	instanceVariableNames: 'messages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Server'!

!ChatMessagesQueue commentStamp: 'FrancoisJarriges 6/2/2016 15:04' prior: 0!
I represent a list of all the messages sent by clients in this chat application.

    Instance Variables
	messages:		<Object>


    Implementation Points!

!ChatMessagesQueue methodsFor: 'adding-removing' stamp: 'FrancoisJarriges 6/1/2016 09:12'!
add: aChatMessage
	messages add: aChatMessage ! !

!ChatMessagesQueue methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/1/2016 09:11'!
initialize

	super initialize.
	messages := OrderedCollection new! !

!ChatMessagesQueue methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/2/2016 16:42'!
listFrom: aChatMessageIndex

	^ (aChatMessageIndex between: 0 and: messages size - 1)
		ifTrue: [ messages copyFrom: aChatMessageIndex + 1 to: messages size ]
		ifFalse: [ #() ]! !

!ChatMessagesQueue methodsFor: 'adding-removing' stamp: 'FrancoisJarriges 6/1/2016 09:12'!
reset
	messages removeAll! !

!ChatMessagesQueue methodsFor: 'accessing' stamp: 'FrancoisJarriges 6/1/2016 09:12'!
size
	^ messages size! !

Object subclass: #ChatServer
	instanceVariableNames: 'teapotServer messagesQueue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LostInMorvanChat-Server'!

!ChatServer commentStamp: 'FrancoisJarriges 6/2/2016 14:47' prior: 0!
I'm a chat server. I'm listening to HTTPRequests and managing a list of messages. 

My clients often request the messages I hold. Indeed, it may happen that client sends a lot of message and another one should request the difference between the messages it knows and the ones published on the server to be able to refresh correctly. 

I register some routes 
	/messages/ aNumber  get all the messages from aNumber to the end.
	/messages/count to get the current number of messages
	/messages/ add to add a message

ChatServer stopAll.
ChatServer startOn: 8080.

ChatClient connect: 'localhost' port: 8080 login: 'toto'.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	messagesQueue:		<Object>
	teapotServer:		<Object>


    Implementation Points!

!ChatServer class methodsFor: 'public' stamp: 'FrancoisJarriges 6/3/2016 15:01'!
startOn: aPortNumber

	^ self new
		initializePort: aPortNumber;
		registerRoutes;
		registerErrorHandlers;
		start;
		yourself! !

!ChatServer class methodsFor: 'public' stamp: 'FrancoisJarriges 6/1/2016 09:44'!
stopAll

	^ self allInstancesDo: #stop! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/3/2016 14:42'!
addMessage: aRequest
	
	messagesQueue 
		add: (STONReader on: aRequest entity string readStream) next ! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/1/2016 09:28'!
initialize

	super initialize.
	messagesQueue := ChatMessagesQueue new! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/3/2016 15:25'!
initializePort: anInteger

	teapotServer := Teapot configure: {
		#defaultOutput -> #text.
		#port -> anInteger.
		#debugMode -> true }
! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/1/2016 09:52'!
messageCount

	^ messagesQueue size! !

!ChatServer methodsFor: 'requests' stamp: 'FrancoisJarriges 6/3/2016 11:54'!
messagesFrom: aRequest

	"^ messagesQueue formattedMessagesFrom: (aRequest at: #id)"
	^ messagesQueue listFrom: (aRequest at: #id)! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/1/2016 10:12'!
registerErrorHandlers
	
	teapotServer 
		exception: KeyNotFound -> (TeaResponse notFound body: 'No such message')! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/3/2016 14:54'!
registerRoutes
	
	teapotServer 
		GET: '/messages/count' -> (Send message: #messageCount to: self);output: #ston;
		GET: '/messages/<id:IsInteger>' -> (Send message: #messagesFrom: to: self); output: #ston;
		POST: '/messages/add' -> (Send message: #addMessage: to: self)! !

!ChatServer methodsFor: 'initialization' stamp: 'FrancoisJarriges 6/3/2016 15:01'!
start
	
	teapotServer start! !

!ChatServer methodsFor: 'public' stamp: 'FrancoisJarriges 6/1/2016 09:41'!
stop

	teapotServer stop.
	messagesQueue reset! !
